@@ -7,10 +7,10 @@ use std::time::Duration;
 use std::time::Instant;
 use std::{fmt, io, thread};
 
-#[cfg(target_arch = "wasm32")]
-use instant::Instant;
 #[cfg(test)]
 use once_cell::sync::Lazy;
+#[cfg(target_arch = "wasm32")]
+use web_time::Instant;
 
 use crate::draw_target::ProgressDrawTarget;
 use crate::state::{AtomicPosition, BarState, ProgressFinish, Reset, TabExpandedString};
@@ -37,13 +37,28 @@ impl fmt::Debug for ProgressBar {
 impl ProgressBar {
     /// Creates a new progress bar with a given length
     ///
-    /// This progress bar by default draws directly to stderr, and refreshes a maximum of 15 times
-    /// a second. To change the refresh rate, set the draw target to one with a different refresh
+    /// This progress bar by default draws directly to stderr, and refreshes a maximum of 20 times
+    /// a second. To change the refresh rate, [set] the [draw target] to one with a different refresh
     /// rate.
+    ///
+    /// [set]: ProgressBar::set_draw_target
+    /// [draw target]: ProgressDrawTarget
     pub fn new(len: u64) -> Self {
         Self::with_draw_target(Some(len), ProgressDrawTarget::stderr())
     }
 
+    /// Creates a new progress bar without a specified length
+    ///
+    /// This progress bar by default draws directly to stderr, and refreshes a maximum of 20 times
+    /// a second. To change the refresh rate, [set] the [draw target] to one with a different refresh
+    /// rate.
+    ///
+    /// [set]: ProgressBar::set_draw_target
+    /// [draw target]: ProgressDrawTarget
+    pub fn no_length() -> Self {
+        Self::with_draw_target(None, ProgressDrawTarget::stderr())
+    }
+
     /// Creates a completely hidden progress bar
     ///
     /// This progress bar still responds to API changes but it does not have a length or render in
@@ -119,7 +134,7 @@ impl ProgressBar {
     /// [`ProgressBarIter`] completes and
     /// [`ProgressBar::is_finished()`] is false.
     /// If you don't want the progress bar to be automatically finished then
-    /// call `on_finish(None)`.
+    /// call `with_finish(Abandon)`.
     ///
     /// [`ProgressBar`]: crate::ProgressBar
     /// [`ProgressBarIter`]: crate::ProgressBarIter
@@ -234,12 +249,12 @@ impl ProgressBar {
     ///
     /// If the progress bar is hidden (e.g. when standard output is not a terminal), `println()`
     /// will not do anything. If you want to write to the standard output in such cases as well, use
-    /// [`suspend`] instead.
+    /// [`ProgressBar::suspend()`] instead.
     ///
     /// If the progress bar was added to a [`MultiProgress`], the log line will be
     /// printed above all other progress bars.
     ///
-    /// [`suspend`]: ProgressBar::suspend
+    /// [`ProgressBar::suspend()`]: ProgressBar::suspend
     /// [`MultiProgress`]: crate::MultiProgress
     pub fn println<I: AsRef<str>>(&self, msg: I) {
         self.state().println(Instant::now(), msg.as_ref());
@@ -260,6 +275,11 @@ impl ProgressBar {
         }
     }
 
+    /// Sets the length of the progress bar to `None`
+    pub fn unset_length(&self) {
+        self.state().unset_length(Instant::now());
+    }
+
     /// Sets the length of the progress bar
     pub fn set_length(&self, len: u64) {
         self.state().set_length(Instant::now(), len);
@@ -290,7 +310,7 @@ impl ProgressBar {
         state.update_estimate_and_draw(Instant::now());
     }
 
-    /// Creates a new weak reference to this `ProgressBar`
+    /// Creates a new weak reference to this [`ProgressBar`]
     pub fn downgrade(&self) -> WeakProgressBar {
         WeakProgressBar {
             state: Arc::downgrade(&self.state),
@@ -375,12 +395,14 @@ impl ProgressBar {
     /// ```
     ///
     /// **Note:** Calling this method on a [`ProgressBar`] linked with a [`MultiProgress`] (after
-    /// running [`MultiProgress::add`]) will unlink this progress bar. If you don't want this
-    /// behavior, call [`MultiProgress::set_draw_target`] instead.
+    /// running [`MultiProgress::add()`]) will unlink this progress bar. If you don't want this
+    /// behavior, call [`MultiProgress::set_draw_target()`] instead.
+    ///
+    /// Use [`ProgressBar::with_draw_target()`] to set the draw target during creation.
     ///
     /// [`MultiProgress`]: crate::MultiProgress
-    /// [`MultiProgress::add`]: crate::MultiProgress::add
-    /// [`MultiProgress::set_draw_target`]: crate::MultiProgress::set_draw_target
+    /// [`MultiProgress::add()`]: crate::MultiProgress::add
+    /// [`MultiProgress::set_draw_target()`]: crate::MultiProgress::set_draw_target
     pub fn set_draw_target(&self, target: ProgressDrawTarget) {
         let mut state = self.state();
         state.draw_target.disconnect(Instant::now());
@@ -391,6 +413,8 @@ impl ProgressBar {
     ///
     /// Useful for external code that writes to the standard output.
     ///
+    /// If the progress bar was added to a [`MultiProgress`], it will suspend the entire [`MultiProgress`].
+    ///
     /// **Note:** The internal lock is held while `f` is executed. Other threads trying to print
     /// anything on the progress bar will be blocked until `f` finishes.
     /// Therefore, it is recommended to avoid long-running operations in `f`.
@@ -402,6 +426,8 @@ impl ProgressBar {
     ///     println!("Log message");
     /// })
     /// ```
+    ///
+    /// [`MultiProgress`]: crate::MultiProgress
     pub fn suspend<F: FnOnce() -> R, R>(&self, f: F) -> R {
         self.state().suspend(Instant::now(), f)
     }
@@ -511,6 +537,28 @@ impl ProgressBar {
         }
     }
 
+    /// Wraps a [`futures::Stream`](https://docs.rs/futures/0.3/futures/stream/trait.StreamExt.html) with the progress bar
+    ///
+    /// ```
+    /// # use indicatif::ProgressBar;
+    /// # futures::executor::block_on(async {
+    /// use futures::stream::{self, StreamExt};
+    /// let pb = ProgressBar::new(10);
+    /// let mut stream = pb.wrap_stream(stream::iter('a'..='z'));
+    ///
+    /// assert_eq!(stream.next().await, Some('a'));
+    /// assert_eq!(stream.count().await, 25);
+    /// # }); // block_on
+    /// ```
+    #[cfg(feature = "futures")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "futures")))]
+    pub fn wrap_stream<S: futures_core::Stream>(&self, stream: S) -> ProgressBarIter<S> {
+        ProgressBarIter {
+            progress: self.clone(),
+            it: stream,
+        }
+    }
+
     /// Returns the current position
     pub fn position(&self) -> u64 {
         self.state().state.pos()
@@ -562,7 +610,7 @@ impl ProgressBar {
     }
 }
 
-/// A weak reference to a `ProgressBar`.
+/// A weak reference to a [`ProgressBar`].
 ///
 /// Useful for creating custom steady tick implementations
 #[derive(Clone, Default)]
@@ -573,15 +621,15 @@ pub struct WeakProgressBar {
 }
 
 impl WeakProgressBar {
-    /// Create a new `WeakProgressBar` that returns `None` when [`upgrade`] is called.
+    /// Create a new [`WeakProgressBar`] that returns `None` when [`upgrade()`] is called.
     ///
-    /// [`upgrade`]: WeakProgressBar::upgrade
+    /// [`upgrade()`]: WeakProgressBar::upgrade
     pub fn new() -> Self {
         Self::default()
     }
 
     /// Attempts to upgrade the Weak pointer to a [`ProgressBar`], delaying dropping of the inner
-    /// value if successful. Returns `None` if the inner value has since been dropped.
+    /// value if successful. Returns [`None`] if the inner value has since been dropped.
     ///
     /// [`ProgressBar`]: struct.ProgressBar.html
     pub fn upgrade(&self) -> Option<ProgressBar> {
@@ -656,7 +704,7 @@ impl TickerControl {
             drop(arc); // Also need to drop Arc otherwise BarState won't be dropped
 
             // Wait for `interval` but return early if we are notified to stop
-            let (_, result) = self
+            let result = self
                 .stopping
                 .1
                 .wait_timeout_while(self.stopping.0.lock().unwrap(), interval, |stopped| {
@@ -665,7 +713,7 @@ impl TickerControl {
                 .unwrap();
 
             // If the wait didn't time out, it means we were notified to stop
-            if !result.timed_out() {
+            if !result.1.timed_out() {
                 break;
             }
         }
